@@ -35,6 +35,9 @@ def _filter_values(vals, vlist=None, must=False):
     if not vlist:  # No value specified equals any value
         return vals
 
+    if vals is None: # cannot iterate over None, return early
+        return vals
+
     if isinstance(vlist, six.string_types):
         vlist = [vlist]
 
@@ -83,34 +86,26 @@ def filter_on_attributes(ava, required=None, optional=None, acs=None,
     """
 
     def _match_attr_name(attr, ava):
-        local_name = None
-
-        for a in ['name_format', 'friendly_name']:
-            _val = attr.get(a)
-            if _val:
-                if a == 'name_format':
-                    local_name = get_local_name(acs, attr['name'], _val)
-                else:
-                    local_name = _val
-                break
-
-        if local_name:
-            _fn = _match(local_name, ava)
-        else:
-            _fn = None
-
-        if not _fn:  # In the unlikely case that someone has provided us with
-            #  URIs as attribute names
-            _fn = _match(attr["name"], ava)
-
+        name = attr["name"].lower()
+        name_format = attr.get("name_format")
+        friendly_name = attr.get("friendly_name")
+        local_name = (
+            get_local_name(acs, name, name_format)
+            or friendly_name
+            or ""
+        )
+        _fn = (
+            _match(local_name, ava)
+            # In the unlikely case that someone has provided us with URIs as attribute names
+            or _match(name, ava)
+        )
         return _fn
 
 
     def _apply_attr_value_restrictions(attr, res, must=False):
-        try:
-            values = [av["text"] for av in attr["attribute_value"]]
-        except KeyError:
-            values = []
+        values = [
+            av["text"] for av in attr.get("attribute_value", [])
+        ]
 
         try:
             res[_fn].extend(_filter_values(ava[_fn], values))
