@@ -1,6 +1,8 @@
 #!/usr/bin/env python
+from saml2.algsupport import algorithm_support_in_metadata
 from saml2.md import AttributeProfile
 from saml2.sigver import security_context
+from saml2.cert import read_cert_from_file
 from saml2.config import Config
 from saml2.validate import valid_instance
 from saml2.time_util import in_a_while
@@ -8,6 +10,7 @@ from saml2.extension import mdui
 from saml2.extension import idpdisc
 from saml2.extension import shibmd
 from saml2.extension import mdattr
+from saml2.extension import sp_type
 from saml2.saml import NAME_FORMAT_URI
 from saml2.saml import AttributeValue
 from saml2.saml import Attribute
@@ -55,58 +58,72 @@ ORG_ATTR_TRANSL = {
 }
 
 MDNS = '"urn:oasis:names:tc:SAML:2.0:metadata"'
+bMDNS = b'"urn:oasis:names:tc:SAML:2.0:metadata"'
 XMLNSXS = " xmlns:xs=\"http://www.w3.org/2001/XMLSchema\""
+bXMLNSXS = b" xmlns:xs=\"http://www.w3.org/2001/XMLSchema\""
 
 
 def metadata_tostring_fix(desc, nspair, xmlstring=""):
     if not xmlstring:
         xmlstring = desc.to_string(nspair)
-    if "\"xs:string\"" in xmlstring and XMLNSXS not in xmlstring:
-        xmlstring = xmlstring.replace(MDNS, MDNS+XMLNSXS)
+
+    try:
+        if "\"xs:string\"" in xmlstring and XMLNSXS not in xmlstring:
+            xmlstring = xmlstring.replace(MDNS, MDNS + XMLNSXS)
+    except TypeError:
+        if b"\"xs:string\"" in xmlstring and bXMLNSXS not in xmlstring:
+            xmlstring = xmlstring.replace(bMDNS, bMDNS + bXMLNSXS)
+
     return xmlstring
 
 
-def create_metadata_string(configfile, config, valid, cert, keyfile, mid, name,
-                           sign):
+def create_metadata_string(
+    configfile,
+    config=None,
+    valid=None,
+    cert=None,
+    keyfile=None,
+    mid=None,
+    name=None,
+    sign=None,
+    sign_alg=None,
+    digest_alg=None,
+):
     valid_for = 0
     nspair = {"xs": "http://www.w3.org/2001/XMLSchema"}
-    #paths = [".", "/opt/local/bin"]
+    # paths = [".", "/opt/local/bin"]
 
     if valid:
         valid_for = int(valid)  # Hours
 
     eds = []
-    if config is not None:
-        eds.append(entity_descriptor(config))
-    else:
+    if config is None:
         if configfile.endswith(".py"):
             configfile = configfile[:-3]
-        config = Config().load_file(configfile, metadata_construction=True)
-        eds.append(entity_descriptor(config))
+        config = Config().load_file(configfile)
+    eds.append(entity_descriptor(config))
 
     conf = Config()
-    conf.key_file = keyfile
-    conf.cert_file = cert
-    conf.debug = 1
+    conf.key_file = config.key_file or keyfile
+    conf.cert_file = config.cert_file or cert
     conf.xmlsec_binary = config.xmlsec_binary
     secc = security_context(conf)
 
+    sign_alg = sign_alg or config.signing_algorithm
+    digest_alg = digest_alg or config.digest_algorithm
     if mid:
-        desc = entities_descriptor(eds, valid_for, name, mid,
-                                   sign, secc)
-        valid_instance(desc)
-
-        return metadata_tostring_fix(desc, nspair)
+        eid, xmldoc = entities_descriptor(
+            eds, valid_for, name, mid, sign, secc, sign_alg, digest_alg
+        )
     else:
         eid = eds[0]
         if sign:
-            eid, xmldoc = sign_entity_descriptor(eid, mid, secc)
+            eid, xmldoc = sign_entity_descriptor(eid, mid, secc, sign_alg, digest_alg)
         else:
             xmldoc = None
 
-        valid_instance(eid)
-        xmldoc = metadata_tostring_fix(eid, nspair, xmldoc)
-        return xmldoc
+    valid_instance(eid)
+    return metadata_tostring_fix(eid, nspair, xmldoc)
 
 
 def _localized_name(val, klass):
@@ -148,46 +165,17 @@ def do_organization_info(ava):
     return org
 
 
-def do_contact_person_info(lava):
-    """ Creates a ContactPerson instance from configuration information"""
+def do_contact_person_info(ava):
+    """Create a ContactPerson instance from configuration information."""
+    cper = md.ContactPerson()
+    cper.loadd(ava)
+    if not cper.contact_type:
+        cper.contact_type = "technical"
+    return cper
 
-    cps = []
-    if lava is None:
-        return cps
-
-    contact_person = md.ContactPerson
-    for ava in lava:
-        cper = md.ContactPerson()
-        for (key, classpec) in contact_person.c_children.values():
-            try:
-                value = ava[key]
-                data = []
-                if isinstance(classpec, list):
-                    # What if value is not a list ?
-                    if isinstance(value, six.string_types):
-                        data = [classpec[0](text=value)]
-                    else:
-                        for val in value:
-                            data.append(classpec[0](text=val))
-                else:
-                    data = classpec(text=value)
-                setattr(cper, key, data)
-            except KeyError:
-                pass
-        for (prop, classpec, _) in contact_person.c_attributes.values():
-            try:
-                # should do a check for valid value
-                setattr(cper, prop, ava[prop])
-            except KeyError:
-                pass
-
-        # ContactType must have a value
-        typ = getattr(cper, "contact_type")
-        if not typ:
-            setattr(cper, "contact_type", "technical")
-
-        cps.append(cper)
 
+def do_contact_persons_info(lava):
+    cps = [do_contact_person_info(ava) for ava in lava]
     return cps
 
 
@@ -232,15 +220,19 @@ def do_key_descriptor(cert=None, enc_cert=None, use="both"):
     return kd_list
 
 
-def do_requested_attribute(attributes, acs, is_required="false"):
+def do_requested_attribute(attributes, acs, is_required="false",
+                           name_format=NAME_FORMAT_URI):
     lista = []
     for attr in attributes:
-        attr = from_local_name(acs, attr, NAME_FORMAT_URI)
+        attr = from_local_name(acs, attr, name_format)
         args = {}
-        for key in attr.keyswv():
-            args[key] = getattr(attr, key)
+        if isinstance(attr, six.string_types):
+            args["name"] = attr
+        else:
+            for key in attr.keyswv():
+                args[key] = getattr(attr, key)
         args["is_required"] = is_required
-        args["name_format"] = NAME_FORMAT_URI
+        args["name_format"] = name_format
         lista.append(md.RequestedAttribute(**args))
     return lista
 
@@ -337,6 +329,7 @@ def do_idpdisc(discovery_response):
     return idpdisc.DiscoveryResponse(index="0", location=discovery_response,
                                      binding=idpdisc.NAMESPACE)
 
+
 ENDPOINTS = {
     "sp": {
         "artifact_resolution_service": (md.ArtifactResolutionService, True),
@@ -399,13 +392,15 @@ def do_extensions(mname, item):
 
 
 def _do_nameid_format(cls, conf, typ):
-    namef = conf.getattr("name_id_format", typ)
-    if namef:
-        if isinstance(namef, six.string_types):
-            ids = [md.NameIDFormat(namef)]
-        else:
-            ids = [md.NameIDFormat(text=form) for form in namef]
-        setattr(cls, "name_id_format", ids)
+    name_id_format = conf.getattr("name_id_format", typ)
+    if not name_id_format:
+        return
+
+    if isinstance(name_id_format, six.string_types):
+        name_id_format = [name_id_format]
+
+    formats = [md.NameIDFormat(text=format) for format in name_id_format]
+    setattr(cls, "name_id_format", formats)
 
 
 def do_endpoints(conf, endpoints):
@@ -416,7 +411,8 @@ def do_endpoints(conf, endpoints):
             servs = []
             i = 1
             for args in conf[endpoint]:
-                if isinstance(args, six.string_types):  # Assume it's the location
+                if isinstance(args,
+                              six.string_types):  # Assume it's the location
                     args = {"location": args,
                             "binding": DEFAULT_BINDING[endpoint]}
                 elif isinstance(args, tuple) or isinstance(args, list):
@@ -444,28 +440,35 @@ def do_endpoints(conf, endpoints):
             pass
     return service
 
+
 DEFAULT = {
     "want_assertions_signed": "true",
     "authn_requests_signed": "false",
     "want_authn_requests_signed": "false",
-    #"want_authn_requests_only_with_valid_cert": "false",
+    # "want_authn_requests_only_with_valid_cert": "false",
 }
 
 
 def do_attribute_consuming_service(conf, spsso):
-
     service_description = service_name = None
     requested_attributes = []
     acs = conf.attribute_converters
     req = conf.getattr("required_attributes", "sp")
+
+    req_attr_name_format = conf.getattr("requested_attribute_name_format", "sp")
+    if req_attr_name_format is None:
+        req_attr_name_format = conf.requested_attribute_name_format
+
     if req:
-        requested_attributes.extend(do_requested_attribute(req, acs,
-                                                           is_required="true"))
+        requested_attributes.extend(
+            do_requested_attribute(req, acs, is_required="true",
+                                   name_format=req_attr_name_format))
 
     opt = conf.getattr("optional_attributes", "sp")
 
     if opt:
-        requested_attributes.extend(do_requested_attribute(opt, acs))
+        requested_attributes.extend(
+            do_requested_attribute(opt, acs, name_format=req_attr_name_format))
 
     try:
         if conf.description:
@@ -541,7 +544,8 @@ def do_spsso_descriptor(conf, cert=None, enc_cert=None):
 
     if cert or enc_cert:
         metadata_key_usage = conf.metadata_key_usage
-        spsso.key_descriptor = do_key_descriptor(cert=cert, enc_cert=enc_cert, use=metadata_key_usage)
+        spsso.key_descriptor = do_key_descriptor(cert=cert, enc_cert=enc_cert,
+                                                 use=metadata_key_usage)
 
     for key in ["want_assertions_signed", "authn_requests_signed"]:
         try:
@@ -589,10 +593,11 @@ def do_idpsso_descriptor(conf, cert=None, enc_cert=None):
         idpsso.extensions.add_extension_element(do_uiinfo(ui_info))
 
     if cert or enc_cert:
-        idpsso.key_descriptor = do_key_descriptor(cert, enc_cert, use=conf.metadata_key_usage)
+        idpsso.key_descriptor = do_key_descriptor(cert, enc_cert,
+                                                  use=conf.metadata_key_usage)
 
     for key in ["want_authn_requests_signed"]:
-                #"want_authn_requests_only_with_valid_cert"]:
+        # "want_authn_requests_only_with_valid_cert"]:
         try:
             val = conf.getattr(key, "idp")
             if val is None:
@@ -619,7 +624,8 @@ def do_aa_descriptor(conf, cert=None, enc_cert=None):
     _do_nameid_format(aad, conf, "aa")
 
     if cert or enc_cert:
-        aad.key_descriptor = do_key_descriptor(cert, enc_cert, use=conf.metadata_key_usage)
+        aad.key_descriptor = do_key_descriptor(cert, enc_cert,
+                                               use=conf.metadata_key_usage)
 
     attributes = conf.getattr("attribute", "aa")
     if attributes:
@@ -648,7 +654,8 @@ def do_aq_descriptor(conf, cert=None, enc_cert=None):
     _do_nameid_format(aqs, conf, "aq")
 
     if cert or enc_cert:
-        aqs.key_descriptor = do_key_descriptor(cert, enc_cert, use=conf.metadata_key_usage)
+        aqs.key_descriptor = do_key_descriptor(cert, enc_cert,
+                                               use=conf.metadata_key_usage)
 
     return aqs
 
@@ -669,24 +676,39 @@ def do_pdp_descriptor(conf, cert=None, enc_cert=None):
     _do_nameid_format(pdp, conf, "pdp")
 
     if cert:
-        pdp.key_descriptor = do_key_descriptor(cert, enc_cert, use=conf.metadata_key_usage)
+        pdp.key_descriptor = do_key_descriptor(cert, enc_cert,
+                                               use=conf.metadata_key_usage)
 
     return pdp
 
 
+def _add_attr_to_entity_attributes(extensions, attribute):
+    try:
+        entity_attributes = next(
+            el
+            for el in extensions.extension_elements
+            if el.tag == mdattr.EntityAttributes.c_tag
+        )
+    except StopIteration:
+        entity_attributes = mdattr.EntityAttributes(attribute=attribute)
+        extensions.add_extension_element(entity_attributes)
+    else:
+        entity_attributes.children.append(attribute)
+
+
 def entity_descriptor(confd):
     mycert = None
     enc_cert = None
     if confd.cert_file is not None:
         mycert = []
-        mycert.append("".join(open(confd.cert_file).readlines()[1:-1]))
+        mycert.append(read_cert_from_file(confd.cert_file))
         if confd.additional_cert_files is not None:
             for _cert_file in confd.additional_cert_files:
-                mycert.append("".join(open(_cert_file).readlines()[1:-1]))
+                mycert.append(read_cert_from_file(_cert_file))
     if confd.encryption_keypairs is not None:
         enc_cert = []
         for _encryption in confd.encryption_keypairs:
-            enc_cert.append("".join(open(_encryption["cert_file"]).readlines()[1:-1]))
+            enc_cert.append(read_cert_from_file(_encryption["cert_file"]))
 
     entd = md.EntityDescriptor()
     entd.entity_id = confd.entityid
@@ -697,14 +719,65 @@ def entity_descriptor(confd):
     if confd.organization is not None:
         entd.organization = do_organization_info(confd.organization)
     if confd.contact_person is not None:
-        entd.contact_person = do_contact_person_info(confd.contact_person)
+        entd.contact_person = do_contact_persons_info(confd.contact_person)
+
+    if confd.entity_attributes:
+        if not entd.extensions:
+            entd.extensions = md.Extensions()
+        attributes = [
+            Attribute(
+                name_format=attr.get("format"),
+                name=attr.get("name"),
+                friendly_name=attr.get("friendly_name"),
+                attribute_value=[
+                    AttributeValue(text=value)
+                    for value in attr.get("values", [])
+                ],
+            )
+            for attr in confd.entity_attributes
+        ]
+        for attribute in attributes:
+            _add_attr_to_entity_attributes(entd.extensions, attribute)
+
+    if confd.assurance_certification:
+        if not entd.extensions:
+            entd.extensions = md.Extensions()
+        ava = [AttributeValue(text=c) for c in confd.assurance_certification]
+        attr = Attribute(
+            attribute_value=ava,
+            name="urn:oasis:names:tc:SAML:attribute:assurance-certification",
+        )
+        _add_attr_to_entity_attributes(entd.extensions, attr)
 
     if confd.entity_category:
-        entd.extensions = md.Extensions()
+        if not entd.extensions:
+            entd.extensions = md.Extensions()
         ava = [AttributeValue(text=c) for c in confd.entity_category]
-        attr = Attribute(attribute_value=ava,
-                         name="http://macedir.org/entity-category")
-        item = mdattr.EntityAttributes(attribute=attr)
+        attr = Attribute(
+            attribute_value=ava, name="http://macedir.org/entity-category"
+        )
+        _add_attr_to_entity_attributes(entd.extensions, attr)
+
+    if confd.entity_category_support:
+        if not entd.extensions:
+            entd.extensions = md.Extensions()
+        ava = [AttributeValue(text=c) for c in confd.entity_category_support]
+        attr = Attribute(
+            attribute_value=ava, name="http://macedir.org/entity-category-support"
+        )
+        _add_attr_to_entity_attributes(entd.extensions, attr)
+
+    for item in algorithm_support_in_metadata(confd.xmlsec_binary):
+        if not entd.extensions:
+            entd.extensions = md.Extensions()
+        entd.extensions.add_extension_element(item)
+
+    conf_sp_type = confd.getattr('sp_type', 'sp')
+    conf_sp_type_in_md = confd.getattr('sp_type_in_metadata', 'sp')
+    if conf_sp_type and conf_sp_type_in_md is True:
+        if not entd.extensions:
+            entd.extensions = md.Extensions()
+        item = sp_type.SPType(text=conf_sp_type)
         entd.extensions.add_extension_element(item)
 
     serves = confd.serves
@@ -720,18 +793,22 @@ def entity_descriptor(confd):
         entd.idpsso_descriptor = do_idpsso_descriptor(confd, mycert, enc_cert)
     if "aa" in serves:
         confd.context = "aa"
-        entd.attribute_authority_descriptor = do_aa_descriptor(confd, mycert, enc_cert)
+        entd.attribute_authority_descriptor = do_aa_descriptor(confd, mycert,
+                                                               enc_cert)
     if "pdp" in serves:
         confd.context = "pdp"
         entd.pdp_descriptor = do_pdp_descriptor(confd, mycert, enc_cert)
     if "aq" in serves:
         confd.context = "aq"
-        entd.authn_authority_descriptor = do_aq_descriptor(confd, mycert, enc_cert)
+        entd.authn_authority_descriptor = do_aq_descriptor(confd, mycert,
+                                                           enc_cert)
 
     return entd
 
 
-def entities_descriptor(eds, valid_for, name, ident, sign, secc):
+def entities_descriptor(
+    eds, valid_for, name, ident, sign, secc, sign_alg=None, digest_alg=None
+):
     entities = md.EntitiesDescriptor(entity_descriptor=eds)
     if valid_for:
         entities.valid_until = in_a_while(hours=valid_for)
@@ -752,7 +829,9 @@ def entities_descriptor(eds, valid_for, name, ident, sign, secc):
             raise SAMLError("If you want to do signing you should define " +
                             "where your public key are")
 
-        entities.signature = pre_signature_part(ident, secc.my_cert, 1)
+        entities.signature = pre_signature_part(ident, secc.my_cert, 1,
+                                                sign_alg=sign_alg,
+                                                digest_alg=digest_alg)
         entities.id = ident
         xmldoc = secc.sign_statement("%s" % entities, class_name(entities))
         entities = md.entities_descriptor_from_string(xmldoc)
@@ -762,7 +841,7 @@ def entities_descriptor(eds, valid_for, name, ident, sign, secc):
     return entities, xmldoc
 
 
-def sign_entity_descriptor(edesc, ident, secc):
+def sign_entity_descriptor(edesc, ident, secc, sign_alg=None, digest_alg=None):
     """
 
     :param edesc: EntityDescriptor instance
@@ -774,7 +853,7 @@ def sign_entity_descriptor(edesc, ident, secc):
     if not ident:
         ident = sid()
 
-    edesc.signature = pre_signature_part(ident, secc.my_cert, 1)
+    edesc.signature = pre_signature_part(ident, secc.my_cert, 1, sign_alg=sign_alg, digest_alg=digest_alg)
     edesc.id = ident
     xmldoc = secc.sign_statement("%s" % edesc, class_name(edesc))
     edesc = md.entity_descriptor_from_string(xmldoc)
