@@ -1,33 +1,22 @@
 #!/usr/bin/env python
+
+import base64
+import hashlib
+import hmac
 import logging
 import random
-
-import time
-import base64
-import six
-import sys
-import hmac
 import string
-
-# from python 2.5
-import imp
+import sys
 import traceback
+import zlib
 
-if sys.version_info >= (2, 5):
-    import hashlib
-else:  # before python 2.5
-    import sha
+import six
 
 from saml2 import saml
 from saml2 import samlp
 from saml2 import VERSION
 from saml2.time_util import instant
 
-try:
-    from hashlib import md5
-except ImportError:
-    from md5 import md5
-import zlib
 
 logger = logging.getLogger(__name__)
 
@@ -169,6 +158,7 @@ def rndstr(size=16, alphabet=""):
         alphabet = string.ascii_letters[0:52] + string.digits
     return type(alphabet)().join(rng.choice(alphabet) for _ in range(size))
 
+
 def rndbytes(size=16, alphabet=""):
     """
     Returns rndstr always as a binary type
@@ -203,10 +193,11 @@ def parse_attribute_map(filenames):
     forward = {}
     backward = {}
     for filename in filenames:
-        for line in open(filename).readlines():
-            (name, friendly_name, name_format) = line.strip().split()
-            forward[(name, name_format)] = friendly_name
-            backward[friendly_name] = (name, name_format)
+        with open(filename) as fp:
+            for line in fp:
+                (name, friendly_name, name_format) = line.strip().split()
+                forward[(name, name_format)] = friendly_name
+                backward[friendly_name] = (name, name_format)
 
     return forward, backward
 
@@ -223,33 +214,46 @@ def identity_attribute(form, attribute, forward_map=None):
     # default is name
     return attribute.name
 
-#----------------------------------------------------------------------------
+# ----------------------------------------------------------------------------
 
 
 def error_status_factory(info):
-    if isinstance(info, Exception):
+    if not isinstance(info, Exception):
+        (status_code_status_code_value, status_message_text) = info
+    else:
         try:
             exc_val = EXCEPTION2STATUS[info.__class__]
         except KeyError:
             exc_val = samlp.STATUS_AUTHN_FAILED
-        try:
-            msg = info.args[0]
-        except IndexError:
-            msg = "%s" % info
-        status = samlp.Status(
-            status_message=samlp.StatusMessage(text=msg),
-            status_code=samlp.StatusCode(
-                value=samlp.STATUS_RESPONDER,
-                status_code=samlp.StatusCode(
-                    value=exc_val)))
-    else:
-        (errcode, text) = info
-        status = samlp.Status(
-            status_message=samlp.StatusMessage(text=text),
-            status_code=samlp.StatusCode(
-                value=samlp.STATUS_RESPONDER,
-                status_code=samlp.StatusCode(value=errcode)))
 
+        try:
+            exc_context = info.args[0]
+            err_ctx = (
+                {"status_message_text": exc_context}
+                if isinstance(exc_context, str)
+                else exc_context
+            )
+        except IndexError:
+            err_ctx = {"status_message_text": str(info)}
+
+        status_message_text = err_ctx.get("status_message_text")
+        status_code_status_code_value = err_ctx.get(
+            "status_code_status_code_value", exc_val
+        )
+
+    status_msg = (
+        samlp.StatusMessage(text=status_message_text)
+        if status_message_text
+        else None
+    )
+
+    status = samlp.Status(
+        status_message=status_msg,
+        status_code=samlp.StatusCode(
+            value=samlp.STATUS_RESPONDER,
+            status_code=samlp.StatusCode(value=status_code_status_code_value),
+        ),
+    )
     return status
 
 
@@ -369,6 +373,9 @@ def do_attribute_statement(identity):
 def factory(klass, **kwargs):
     instance = klass()
     for key, val in kwargs.items():
+        if isinstance(val, dict):
+            cls = instance.child_class(key)
+            val = factory(cls, **val)
         setattr(instance, key, val)
     return instance
 
@@ -384,10 +391,7 @@ def signature(secret, parts):
             part = part.encode('utf-8')
         newparts.append(part)
     parts = newparts
-    if sys.version_info >= (2, 5):
-        csum = hmac.new(secret, digestmod=hashlib.sha1)
-    else:
-        csum = hmac.new(secret, digestmod=sha)
+    csum = hmac.new(secret, digestmod=hashlib.sha1)
 
     for part in parts:
         csum.update(part)
@@ -403,67 +407,6 @@ def verify_signature(secret, parts):
         return False
 
 
-FTICKS_FORMAT = "F-TICKS/SWAMID/2.0%s#"
-
-
-def fticks_log(sp, logf, idp_entity_id, user_id, secret, assertion):
-    """
-    'F-TICKS/' federationIdentifier '/' version *('#' attribute '=' value) '#'
-    Allowed attributes:
-        TS	the login time stamp
-        RP	the relying party entityID
-        AP	the asserting party entityID (typcially the IdP)
-        PN	a sha256-hash of the local principal name and a unique key
-        AM	the authentication method URN
-
-    :param sp: Client instance
-    :param logf: The log function to use
-    :param idp_entity_id: IdP entity ID
-    :param user_id: The user identifier
-    :param secret: A salt to make the hash more secure
-    :param assertion: A SAML Assertion instance gotten from the IdP
-    """
-    csum = hmac.new(secret, digestmod=hashlib.sha1)
-    csum.update(user_id)
-    ac = assertion.AuthnStatement[0].AuthnContext[0]
-
-    info = {
-        "TS": time.time(),
-        "RP": sp.entity_id,
-        "AP": idp_entity_id,
-        "PN": csum.hexdigest(),
-        "AM": ac.AuthnContextClassRef.text
-    }
-    logf.info(FTICKS_FORMAT % "#".join(["%s=%s" % (a, v) for a, v in info]))
-
-
-def dynamic_importer(name, class_name=None):
-    """
-    Dynamically imports modules / classes
-    """
-    try:
-        fp, pathname, description = imp.find_module(name)
-    except ImportError:
-        print("unable to locate module: " + name)
-        return None, None
-
-    try:
-        package = imp.load_module(name, fp, pathname, description)
-    except Exception:
-        raise
-
-    if class_name:
-        try:
-            _class = imp.load_module("%s.%s" % (name, class_name), fp,
-                                     pathname, description)
-        except Exception:
-            raise
-
-        return package, _class
-    else:
-        return package, None
-
-
 def exception_trace(exc):
     message = traceback.format_exception(*sys.exc_info())
 
