@@ -15,6 +15,8 @@
 
 
 import base64
+from datetime import date
+from datetime import datetime
 
 from saml2.validate import valid_ipv4, MustValueError
 from saml2.validate import valid_ipv6
@@ -41,7 +43,7 @@ XSI_NAMESPACE = 'http://www.w3.org/2001/XMLSchema-instance'
 NS_SOAP_ENC = "http://schemas.xmlsoap.org/soap/encoding/"
 # type definitions for xmlschemas
 XSI_TYPE = '{%s}type' % XSI_NAMESPACE
-# nil type definition for xmlschemas 
+# nil type definition for xmlschemas
 XSI_NIL = '{%s}nil' % XSI_NAMESPACE
 
 # idp and sp communicate usually about a subject(NameID)
@@ -89,7 +91,7 @@ NAMEID_FORMATS_SAML2 = (
 # The specification was later updated with errata, and the new version is here:
 # https://www.oasis-open.org/committees/download.php/56782/sstc-saml-profiles-errata-2.0-wd-07.pdf
 
-# XML based values for SAML attributes  
+# XML based values for SAML attributes
 PROFILE_ATTRIBUTE_BASIC = (
     "urn:oasis:names:tc:SAML:2.0:profiles:attribute:basic")
 
@@ -307,6 +309,11 @@ class AttributeValueBase(SamlBase):
                 }[str(x).lower()],
                 'to_text': lambda x: str(x).lower(),
             },
+            'date': {
+                'type': date,
+                'to_type': lambda x: datetime.strptime(x, '%Y-%m-%d').date(),
+                'to_text': str,
+            },
             'base64Binary': {
                 'type': str,
                 'to_type': str,
@@ -327,9 +334,10 @@ class AttributeValueBase(SamlBase):
         }
 
         xsd_string = (
-            'base64Binary' if base64encode
-            else self.get_type()
-            or type_to_xsd.get(type(value)))
+            'base64Binary'
+            if base64encode
+            else self.get_type() or type_to_xsd.get(type(value))
+        )
 
         xsd_ns, xsd_type = (
             ['', type(None)] if xsd_string is None
@@ -340,7 +348,10 @@ class AttributeValueBase(SamlBase):
             ] if ':' not in xsd_string
             else xsd_string.split(':', 1))
 
-        xsd_type_props = xsd_types_props.get(xsd_type, {})
+        xsd_type_props = xsd_types_props.get(xsd_type)
+        if not xsd_type_props:
+            xsd_type_props = xsd_types_props.get("string")
+
         valid_type = xsd_type_props.get('type', type(None))
         to_type = xsd_type_props.get('to_type', str)
         to_text = xsd_type_props.get('to_text', str)
@@ -349,7 +360,7 @@ class AttributeValueBase(SamlBase):
         if type(value) is str and valid_type is not str:
             try:
                 value = to_type(value)
-            except (TypeError, ValueError, KeyError) as e:
+            except (TypeError, ValueError, KeyError):
                 # the cast failed
                 _wrong_type_value(xsd=xsd_type, value=value)
 
