@@ -1,14 +1,16 @@
 # coding=utf-8
-import pytest
+import copy
 
 from saml2.argtree import add_path
 from saml2.authn_context import pword
 from saml2.mdie import to_dict
-from saml2 import md, assertion
+from saml2 import md, assertion, create_class_from_xml_string, config
+from saml2.mdstore import MetadataStore
 from saml2.saml import Attribute
 from saml2.saml import Issuer
 from saml2.saml import NAMEID_FORMAT_ENTITY
 from saml2.saml import NAME_FORMAT_URI
+from saml2.saml import NAME_FORMAT_UNSPECIFIED
 from saml2.saml import AttributeValue
 from saml2.saml import NameID
 from saml2.saml import NAMEID_FORMAT_TRANSIENT
@@ -21,20 +23,28 @@ from saml2.s_utils import MissingValue
 from saml2 import attribute_converter
 from saml2.attribute_converter import ac_factory, AttributeConverterNOOP
 
-from py.test import raises
+from pytest import raises
 
 from saml2.extension import mdui
 from saml2.extension import idpdisc
 from saml2.extension import dri
 from saml2.extension import mdattr
-from saml2.extension import ui
 from saml2 import saml
 from saml2 import xmldsig
 from saml2 import xmlenc
 
 from pathutils import full_path
 
-ONTS = [saml, mdui, mdattr, dri, ui, idpdisc, md, xmldsig, xmlenc]
+ONTS = [saml, mdui, mdattr, dri, idpdisc, md, xmldsig, xmlenc]
+ATTRCONV = ac_factory(full_path("attributemaps"))
+sec_config = config.Config()
+
+METADATACONF = {
+    "1": [{
+        "class": "saml2.mdstore.MetaDataFile",
+        "metadata": [(full_path("swamid-2.0.xml"),)],
+    }],
+}
 
 
 def _eq(l1, l2):
@@ -58,15 +68,55 @@ mail = to_dict(md.RequestedAttribute(name="urn:oid:0.9.2342.19200300.100.1.3",
 
 
 def test_filter_on_attributes_0():
-    a = to_dict(Attribute(name="urn:oid:2.5.4.5", name_format=NAME_FORMAT_URI,
-                          friendly_name="serialNumber"), ONTS)
+    a = to_dict(
+        Attribute(
+            name="urn:oid:2.5.4.5",
+            name_format=NAME_FORMAT_URI,
+            friendly_name="serialNumber",
+        ),
+        ONTS,
+    )
 
-    required = [a]
-    ava = {"serialNumber": ["12345"]}
+    b = to_dict(
+        Attribute(
+            name="urn:oasis:names:tc:SAML:attribute:subject-id",
+            name_format=NAME_FORMAT_URI,
+            friendly_name="subject-id",
+        ),
+        ONTS,
+    )
+
+    c = to_dict(
+        Attribute(
+            name="unmapped_attr_name",
+            name_format=NAME_FORMAT_URI,
+            friendly_name="unmapped attr name",
+        ),
+        ONTS,
+    )
+
+    d = to_dict(
+        Attribute(
+            name="urn:oid:2.5.4.6",
+            friendly_name="c",
+        ),
+        ONTS,
+    )
+
+    required = [a, b, c, d]
+    ava = {
+        "serialNumber": ["12345"],
+        "subject-id": ["id_12345"],
+        "unmapped_attr_name": ["abcd"],
+        "c": ["some-country"],
+    }
 
     ava = filter_on_attributes(ava, required, acs=ac_factory())
-    assert list(ava.keys()) == ["serialNumber"]
+    assert set(ava.keys()) == {"serialNumber", "subject-id", "unmapped_attr_name", "c"}
     assert ava["serialNumber"] == ["12345"]
+    assert ava["subject-id"] == ["id_12345"]
+    assert ava["unmapped_attr_name"] == ["abcd"]
+    assert ava["c"] == ["some-country"]
 
 
 def test_filter_on_attributes_1():
@@ -82,7 +132,7 @@ def test_filter_on_attributes_1():
 
 
 def test_filter_on_attributes_2():
-    
+
     a = to_dict(Attribute(friendly_name="surName",name="urn:oid:2.5.4.4",
                           name_format=NAME_FORMAT_URI), ONTS)
     required = [a]
@@ -117,7 +167,7 @@ def test_filter_on_attributes_with_missing_required_attribute():
         friendly_name="eduPersonTargetedID",
         name="urn:oid:1.3.6.1.4.1.5923.1.1.1.10",
         name_format=NAME_FORMAT_URI), ONTS)
-    with pytest.raises(MissingValue):
+    with raises(MissingValue):
         filter_on_attributes(ava, required=[eptid], acs=ac_factory())
 
 
@@ -154,7 +204,7 @@ def test_lifetime_1():
             "attribute_restrictions": {
                 "givenName": None,
                 "surName": None,
-                "mail": [".*@.*\.umu\.se"],
+                "mail": [r".*@.*\.umu\.se"],
             }
         }}
 
@@ -176,7 +226,7 @@ def test_lifetime_2():
             "attribute_restrictions": {
                 "givenName": None,
                 "surName": None,
-                "mail": [".*@.*\.umu\.se"],
+                "mail": [r".*@.*\.umu\.se"],
             }
         }}
 
@@ -198,7 +248,7 @@ def test_ava_filter_1():
             "attribute_restrictions": {
                 "givenName": None,
                 "surName": None,
-                "mail": [".*@.*\.umu\.se"],
+                "mail": [r".*@.*\.umu\.se"],
             }
         }}
 
@@ -208,7 +258,7 @@ def test_ava_filter_1():
            "surName": "Jeter",
            "mail": "derek@example.com"}
 
-    ava = r.filter(ava, "urn:mace:umu.se:saml:roland:sp", None, None)
+    ava = r.filter(ava, "urn:mace:umu.se:saml:roland:sp")
     assert _eq(list(ava.keys()), ["givenName", "surName"])
 
     ava = {"givenName": "Derek",
@@ -228,7 +278,7 @@ def test_ava_filter_2():
             "attribute_restrictions": {
                 "givenName": None,
                 "sn": None,
-                "mail": [".*@.*\.umu\.se"],
+                "mail": [r".*@.*\.umu\.se"],
             }
         }}
 
@@ -237,8 +287,7 @@ def test_ava_filter_2():
     ava = {"givenName": "Derek", "sn": "Jeter", "mail": "derek@example.com"}
 
     # mail removed because it doesn't match the regular expression
-    _ava = policy.filter(ava, 'urn:mace:umu.se:saml:roland:sp', None, [mail],
-                         [gn, sn])
+    _ava = policy.filter(ava, 'urn:mace:umu.se:saml:roland:sp', required=[mail], optional=[gn, sn])
 
     assert _eq(sorted(list(_ava.keys())), ["givenName", 'sn'])
 
@@ -246,8 +295,7 @@ def test_ava_filter_2():
 
     # it wasn't there to begin with
     try:
-        policy.filter(ava, 'urn:mace:umu.se:saml:roland:sp', None,
-                      [gn, sn, mail])
+        policy.filter(ava, 'urn:mace:umu.se:saml:roland:sp', required=[gn, sn, mail])
     except MissingValue:
         pass
 
@@ -264,7 +312,7 @@ def test_ava_filter_dont_fail():
             "attribute_restrictions": {
                 "givenName": None,
                 "surName": None,
-                "mail": [".*@.*\.umu\.se"],
+                "mail": [r".*@.*\.umu\.se"],
             },
             "fail_on_missing_requested": False
         }}
@@ -277,8 +325,7 @@ def test_ava_filter_dont_fail():
 
     # mail removed because it doesn't match the regular expression
     # So it should fail if the 'fail_on_ ...' flag wasn't set
-    _ava = policy.filter(ava, 'urn:mace:umu.se:saml:roland:sp', None,
-                         [mail], [gn, sn])
+    _ava = policy.filter(ava, 'urn:mace:umu.se:saml:roland:sp', required=[mail], optional=[gn, sn])
 
     assert _ava
 
@@ -286,8 +333,7 @@ def test_ava_filter_dont_fail():
            "surName": "Jeter"}
 
     # it wasn't there to begin with
-    _ava = policy.filter(ava, 'urn:mace:umu.se:saml:roland:sp',
-                         None, [gn, sn, mail])
+    _ava = policy.filter(ava, 'urn:mace:umu.se:saml:roland:sp', required=[gn, sn, mail])
 
     assert _ava
 
@@ -441,7 +487,8 @@ def test_filter_values_req_2():
     required = [a1, a2]
     ava = {"serialNumber": ["12345"], "givenName": ["Lars"]}
 
-    raises(MissingValue, filter_on_attributes, ava, required, acs=ac_factory())
+    with raises(MissingValue):
+        filter_on_attributes(ava, required, acs=ac_factory())
 
 
 def test_filter_values_req_3():
@@ -467,7 +514,8 @@ def test_filter_values_req_4():
     required = [a]
     ava = {"serialNumber": ["12345"]}
 
-    raises(MissingValue, filter_on_attributes, ava, required, acs=ac_factory())
+    with raises(MissingValue):
+        filter_on_attributes(ava, required, acs=ac_factory())
 
 
 def test_filter_values_req_5():
@@ -564,7 +612,8 @@ def test_filter_values_req_opt_2():
     ava = {"surname": ["Hedberg"], "givenName": ["Roland"],
            "eduPersonAffiliation": ["staff"], "uid": ["rohe0002"]}
 
-    raises(MissingValue, "filter_on_attributes(ava, r, o, acs=ac_factory())")
+    with raises(MissingValue):
+        filter_on_attributes(ava, r, o, acs=ac_factory())
 
 
 # ---------------------------------------------------------------------------
@@ -620,7 +669,7 @@ def test_filter_ava_0():
            "mail": ["derek@nyy.mlb.com"]}
 
     # No restrictions apply
-    ava = policy.filter(ava, "urn:mace:example.com:saml:roland:sp", [], [])
+    ava = policy.filter(ava, "urn:mace:example.com:saml:roland:sp")
 
     assert _eq(sorted(list(ava.keys())), ["givenName", "mail", "surName"])
     assert ava["givenName"] == ["Derek"]
@@ -647,7 +696,7 @@ def test_filter_ava_1():
            "mail": ["derek@nyy.mlb.com"]}
 
     # No restrictions apply
-    ava = policy.filter(ava, "urn:mace:example.com:saml:roland:sp", [], [])
+    ava = policy.filter(ava, "urn:mace:example.com:saml:roland:sp")
 
     assert _eq(sorted(list(ava.keys())), ["givenName", "surName"])
     assert ava["givenName"] == ["Derek"]
@@ -672,7 +721,7 @@ def test_filter_ava_2():
            "mail": ["derek@nyy.mlb.com"]}
 
     # No restrictions apply
-    ava = policy.filter(ava, "urn:mace:example.com:saml:roland:sp", [], [])
+    ava = policy.filter(ava, "urn:mace:example.com:saml:roland:sp")
 
     assert _eq(list(ava.keys()), ["mail"])
     assert ava["mail"] == ["derek@nyy.mlb.com"]
@@ -688,7 +737,7 @@ def test_filter_ava_3():
         "urn:mace:example.com:saml:roland:sp": {
             "lifetime": {"minutes": 5},
             "attribute_restrictions": {
-                "mail": [".*@example\.com$"],
+                "mail": [r".*@example\.com$"],
             }
         }})
 
@@ -696,7 +745,7 @@ def test_filter_ava_3():
            "mail": ["derek@nyy.mlb.com", "dj@example.com"]}
 
     # No restrictions apply
-    ava = policy.filter(ava, "urn:mace:example.com:saml:roland:sp", [], [])
+    ava = policy.filter(ava, "urn:mace:example.com:saml:roland:sp")
 
     assert _eq(list(ava.keys()), ["mail"])
     assert ava["mail"] == ["dj@example.com"]
@@ -712,7 +761,7 @@ def test_filter_ava_4():
         "urn:mace:example.com:saml:roland:sp": {
             "lifetime": {"minutes": 5},
             "attribute_restrictions": {
-                "mail": [".*@example\.com$"],
+                "mail": [r".*@example\.com$"],
             }
         }})
 
@@ -720,7 +769,7 @@ def test_filter_ava_4():
            "mail": ["derek@nyy.mlb.com", "dj@example.com"]}
 
     # No restrictions apply
-    ava = policy.filter(ava, "urn:mace:example.com:saml:curt:sp", [], [])
+    ava = policy.filter(ava, "urn:mace:example.com:saml:curt:sp")
 
     assert _eq(sorted(list(ava.keys())), ['mail', 'givenName', 'surName'])
     assert _eq(ava["mail"], ["derek@nyy.mlb.com", "dj@example.com"])
@@ -759,7 +808,7 @@ def test_req_opt():
            'uid': 'rohe0002', 'edupersonaffiliation': 'staff'}
 
     sp_entity_id = "urn:mace:example.com:saml:curt:sp"
-    fava = policy.filter(ava, sp_entity_id, None, req, opt)
+    fava = policy.filter(ava, sp_entity_id, required=req, optional=opt)
     assert fava
 
 
@@ -858,37 +907,89 @@ def test_assertion_with_noop_attribute_conv():
             assert attr.attribute_value[0].text == "Roland"
 
 
-# THis test doesn't work without a MetadataStore instance
-# def test_filter_ava_5():
-#    policy = Policy({
-#        "default": {
-#            "lifetime": {"minutes": 15},
-#            #"attribute_restrictions": None  # means all I have
-#            "entity_categories": ["swamid", "edugain"]
-#        }
-#    })
-#
-#    ava = {"givenName": ["Derek"], "surName": ["Jeter"],
-#           "mail": ["derek@nyy.mlb.com", "dj@example.com"]}
-#
-#    ava = policy.filter(ava, "urn:mace:example.com:saml:curt:sp", None, [], [])
-#
-#    # using entity_categories means there *always* are restrictions
-#    # in this case the only allowed attribute is eduPersonTargetedID
-#    # which isn't available in the ava hence zip is returned.
-#    assert ava == {}
+def test_filter_ava_5():
+    mds = MetadataStore(ATTRCONV, sec_config, disable_ssl_certificate_validation=True)
+    mds.imp(METADATACONF["1"])
+
+    policy_conf = {
+        "default": {
+            "lifetime": {"minutes": 15},
+            "attribute_restrictions": None,  # means all I have
+            "entity_categories": ["swamid", "edugain"]
+        }
+    }
+    policy = Policy(restrictions=policy_conf, mds=mds)
+
+    ava = {
+        "givenName": ["Derek"],
+        "surName": ["Jeter"],
+        "mail": [
+            "derek@nyy.mlb.com",
+            "dj@example.com",
+        ],
+    }
+    ava = policy.filter(ava, "urn:mace:example.com:saml:curt:sp")
+
+    # using entity_categories means there *always* are restrictions
+    # in this case the only allowed attribute is eduPersonTargetedID
+    # which isn't available in the ava hence zip is returned.
+    assert ava == {}
+
+
+def test_filter_ava_registration_authority_1():
+    mds = MetadataStore(ATTRCONV, sec_config, disable_ssl_certificate_validation=True)
+    mds.imp(METADATACONF["1"])
+
+    policy_conf = {
+        "default": {
+            "lifetime": {"minutes": 15},
+            "attribute_restrictions": None,
+        },
+        "http://rr.aai.switch.ch/": {
+            "attribute_restrictions": {
+                "givenName": None,
+                "surName": None,
+            }
+        }
+    }
+    policy = Policy(restrictions=policy_conf, mds=mds)
+
+    attributes = {
+        "givenName": ["Derek"],
+        "surName": ["Jeter"],
+        "mail": [
+            "derek@nyy.mlb.com",
+            "dj@example.com",
+        ],
+    }
+
+    # SP registered with http://rr.aai.switch.ch/
+    ava = policy.filter(attributes, "https://aai-idp.unibe.ch/idp/shibboleth")
+    assert _eq(sorted(list(ava.keys())), ["givenName", "surName"])
+    assert ava["givenName"] == ["Derek"]
+    assert ava["surName"] == ["Jeter"]
+
+    # SP not registered with http://rr.aai.switch.ch/
+    ava = policy.filter(attributes, "https://alpha.kib.ki.se/shibboleth")
+    assert _eq(sorted(list(ava.keys())), ["givenName", "mail", "surName"])
+    assert ava["givenName"] == ["Derek"]
+    assert ava["surName"] == ["Jeter"]
+    assert ava["mail"] == ["derek@nyy.mlb.com", "dj@example.com"]
 
 
 def test_assertion_with_zero_attributes():
     ava = {}
     ast = Assertion(ava)
-    policy = Policy({
+
+    policy_conf = {
         "default": {
             "lifetime": {"minutes": 240},
             "attribute_restrictions": None,  # means all I have
             "name_form": NAME_FORMAT_URI
         },
-    })
+    }
+    policy = Policy(policy_conf)
+
     name_id = NameID(format=NAMEID_FORMAT_TRANSIENT, text="foobar")
     issuer = Issuer(text="entityid", format=NAMEID_FORMAT_ENTITY)
     farg = add_path(
@@ -942,6 +1043,21 @@ def test_assertion_with_authn_instant():
     assert msg.authn_statement[0].authn_instant == "2009-02-13T23:31:30Z"
 
 
+def test_attribute_producer_should_default_to_uri():
+    attr = Attribute()
+    assert attr.name_format == NAME_FORMAT_URI
+
+
+def test_attribute_consumer_should_default_to_unspecified():
+    attr_str = """
+      <saml:Attribute Name="uid" xmlns:saml="urn:oasis:names:tc:SAML:2.0:assertion" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
+        <saml:AttributeValue xsi:type="xs:string">test</saml:AttributeValue>
+      </saml:Attribute>
+    """
+    attr = create_class_from_xml_string(Attribute, attr_str)
+    assert attr.name_format == NAME_FORMAT_UNSPECIFIED
+
+
 if __name__ == "__main__":
     test_assertion_2()
 
