@@ -1,4 +1,4 @@
-import xmldsig as ds
+from saml2 import xmldsig as ds
 from saml2 import config
 from saml2 import extension_elements_to_elements
 from saml2 import element_to_extension_element
@@ -9,6 +9,7 @@ from saml2.extension.pefim import SPCertEnc
 from saml2.samlp import Extensions
 from saml2.samlp import authn_request_from_string
 from saml2.sigver import read_cert_from_file
+from pathutils import full_path
 
 __author__ = 'roland'
 
@@ -17,7 +18,7 @@ conf.load_file("server_conf")
 client = Saml2Client(conf)
 
 # place a certificate in an authn request
-cert = read_cert_from_file("test.pem", "pem")
+cert = read_cert_from_file(full_path("test.pem"))
 
 spcertenc = SPCertEnc(
     x509_data=ds.X509Data(
@@ -34,7 +35,7 @@ req_id, req = client.create_authn_request(
     extensions=extensions)
 
 
-print req
+print(req)
 
 # Get a certificate from an authn request
 
@@ -47,5 +48,5 @@ _elem = extension_elements_to_elements(parsed.extensions.extension_elements,
 
 assert len(_elem) == 1
 _spcertenc = _elem[0]
-_cert = _spcertenc.x509_data[0].x509_certificate.text
-assert cert == _cert
\ No newline at end of file
+_cert = _spcertenc.key_info[0].x509_data[0].x509_certificate.text
+assert cert == _cert
